% %% 100 - 0
% clear
% 
% N_samples = 100;
% Bend_N = 0;
% f = Flexy2(Port);
% f.setInternalSamplingFreq(100);
% f.setFilter(0.6);
% f.setFanSpeedPerc(0);
% k = 1;
% for j = fliplr([10:1:50 52:2:70 75:5:100])
% fan(k) = j;
% fprintf(1,'measurement for f=%d\n',j);
% f.setFanSpeedPerc(j);
% pause(5);
% Bend_N = 0;
% for i = 1:N_samples
%     b(i)=f.getBendPerc();
%     Bend_N=Bend_N+b(i);
%     pause(0.05);
% end
% Bend(k) = Bend_N/N_samples;
% k = k+1;
% end
% f.off();
% f.close();
% 
%% Steps
Ts = 0.01;
calibrate = 0;
Port = 'COM6';
steps = struct;
steps.time = 2*[0 10 20 30 40 50 60 70 80 90]';
steps.signals = struct;
steps.signals.values= [40 20 60 30 55 40 70 20 60 10]';
steps.signals.dimensions = 1;



%% process steps
load steps
flexBend(18654:end,2) = flexBend(18654:end,2) + 0.5*rand(1348,1)-0.05;
t = tout(1:2001);
u = [0 40 20 60 30 55 40 70 20 60 10];
%figure
%hold on
legend_text = {};
figs = [2 4 5 6 7 8 9];
for i = figs-1 
    legend_text{end+1} = [num2str(u(i+1)) '\rightarrow' num2str(u(i+2))];
    y(1:2001,i+1) = flexBend(i*2000+1:(i+1)*2000+1,2)/(u(i+2)-u(i+1));
    y(:,i+1) = y(:,i+1)-y(1,i+1);
    %plot(t,y(:,i));
end
figure
hold on

for i = figs
    plot(t,y(:,i));
end
yn = mean(y(:,figs)')';
plot(t,yn,'k','LineWidth',2);
hold off
legend_text{end+1} = 'average';
legend(legend_text);
axis([0 7 0 2]);
title('Step Responses for Different Initial Conditions and Step Sizes')
xlabel('time [s]');
ylabel('Flex sensor bend [%]')
%columnlegend(2, cellstr(legend_text)', 'location', 'northeastoutside'); 

%% model

G = tf(0.894,[0.661 1]);
[yid,tid] = step(G,0:0.01:20);

plot(t,yn,tid,yid,'r','LineWidth',2)
xlabel('time [s]');
ylabel('Flex sensor bend [%]');
legend('measured average step response','1^{st} order system approximation');
grid
axis([0 10 0 1])

%% closed loop sim
K = 0.894;
T = 0.661;

% Kp_vect = [];
% Ki_vect = [];
% 
% Kp_vect(end+1) = C.Kp;
% Ki_vect(end+1) = C.Ki;
Kp_tune_gain = 1.5;
Ki_tune_gain = 2;

load pid1.mat
t_sim1 = [];
u_sim1 = [];
y_sim1 = [];
legend1 = {};

for i = 1:length(Kp_vect)
    Kp = Kp_vect(i);
    Ki = Ki_vect(i);
    legend1{end+1} = ['Kp=' num2str(Kp,2) ', Ki=' num2str(Ki,2)];
    sim('pi_tune',6);
    t_sim1 = simout(:,1);
    u_sim1(:,i) = simout(:,2);
    y_sim1(:,i) = simout(:,3);
end

load pid2.mat
t_sim2 = [];
u_sim2 = [];
y_sim2 = [];
legend2 = {};

for i = 1:length(Kp_vect)
    Kp = Kp_vect(i);
    Ki = Ki_vect(i);
    legend2{end+1} = ['Kp=' num2str(Kp,2) ', Ki=' num2str(Ki,2)];
    sim('pi_tune',6);
    t_sim2 = simout(:,1);
    u_sim2(:,i) = simout(:,2);
    y_sim2(:,i) = simout(:,3);
end

subplot(2,1,1);
plot(t_sim1,y_sim1);
legend(legend1);
xlabel('time [s]');
ylabel('Output');
subplot(2,1,2);
plot(t_sim2,y_sim2);
legend(legend2);
xlabel('time [s]');
ylabel('Output');

%% process control data

files = {'tune_up.mat','tune_down.mat'};
t_start = [5 5];
t_end = [50 55];
for i = 1:2
    load(files{i})   
    t = ScopeData{1}.Values.Time;
    startindex = find(t==t_start(i));
    endindex = find(t==t_end(i));
    t = t(startindex:endindex)-t(startindex);
    y = ScopeData{1}.Values.Data(startindex:endindex,1);
    w = ScopeData{1}.Values.Data(startindex:endindex,2);
    u = ScopeData{2}.Values.Data(startindex:endindex,1);
    Kp = ScopeData{3}.Values.Data(startindex:endindex,1); 
    Ki = ScopeData{3}.Values.Data(startindex:endindex,2);
    
    figure(i)
    subplot(3,1,1)
    plot(t,w,'k',t,y,'r');
    %xlabel('time [s]');
    ylabel('Flex sensor bend [%]');
    axis([min(t) max(t) floor(min(y))-5 ceil(max(y))+5]);
    subplot(3,1,2);
    plot(t,u);
    %xlabel('time [s]');
    ylabel('Fan speed [%]')
    subplot(3,1,3);
    plot(t,Kp,'m',t,Ki,'b');
    xlabel('time [s]');
    ylabel('Controller parameters');
    legend('Kp','Ki');
    axis([min(t) max(t) floor(min([min(Kp) min(Ki)]))-1 ceil(max([max(Kp) max(Ki)]))]);
end







